import logging
from django.db import models
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.hashers import make_password, check_password
from shared_models.common.base import BaseModel
from shared_models.common.utils import generate_referral_code
from encrypted_model_fields.fields import EncryptedEmailField, EncryptedCharField
from shared_models.common.mixins import SensitiveFieldsTrackingMixin
import hashlib
from .enums import KycStatus, IbStatus, LeadStatus # Import the new enums

logger = logging.getLogger(__name__)

class Customer(SensitiveFieldsTrackingMixin, BaseModel):
    # Sensitive fields with encryption
    id = models.AutoField(primary_key=True, editable=False)
    email = models.EmailField(_('email address'), unique=True)
    first_name = models.CharField(max_length=150, blank=True)
    last_name = models.CharField(max_length=150, blank=True)
    phone = models.CharField(max_length=20, blank=True)
    mobile = models.CharField(max_length=20, blank=True)
    
    # Define sensitive fields for tracking
    sensitive_fields = []
    requires_approval_fields = []  # Changes to email require approval
    
    # Your existing fields
    country = models.CharField(max_length=20, blank=True)
    nationality = models.CharField(max_length=2, blank=True)
    source = models.CharField(max_length=20, blank=True)

    # IB Relationships
    ib = models.ForeignKey('customers.Customer', on_delete=models.SET_NULL, null=True, blank=True, related_name='referred_customers')
    agreement_id = models.CharField(max_length=100, blank=True)

    # Account Status
    is_funded = models.BooleanField(default=False)
    first_deposit = models.DateTimeField(null=True)
    is_individual = models.BooleanField(default=True)
    is_lead = models.BooleanField(default=False)
    is_client = models.BooleanField(default=False)
    is_ib = models.BooleanField(default=False)
    is_real = models.BooleanField(default=False)
    is_live = models.BooleanField(default=False)
    is_corporate = models.BooleanField(default=False)
    
    # Forms
    submit_individual_form = models.BooleanField(default=False)
    submit_corporate_form = models.BooleanField(default=False)
    submit_ib_form = models.BooleanField(default=False)
    
    # KYC Status - Using Enum
    kyc_status = models.CharField(max_length=20, choices=KycStatus.choices, default=KycStatus.NOT_UPLOADED)
    level_one_kyc_status = models.CharField(max_length=20, choices=KycStatus.choices, default=KycStatus.NOT_UPLOADED)
    level_two_kyc_status = models.CharField(max_length=20, choices=KycStatus.choices, default=KycStatus.NOT_UPLOADED)
    level_three_kyc_status = models.CharField(max_length=20, choices=KycStatus.choices, default=KycStatus.NOT_UPLOADED)
    level_three_kyc_required = models.BooleanField(default=False)
    level_one_approved_date = models.DateTimeField(null=True)
    level_two_approved_date = models.DateTimeField(null=True)
    level_three_approved_date = models.DateTimeField(null=True)

    # IB Status - Using Enum
    ib_status = models.CharField(max_length=20, choices=IbStatus.choices, null=True, blank=True)
    
    # Trading Status
    start_trading = models.BooleanField(default=False)
    last_active = models.DateTimeField(null=True)
    last_transaction = models.DateTimeField(null=True)
    last_trade = models.DateTimeField(null=True)

    # Lead Status - Using Enum
    lead_status = models.CharField(max_length=20, choices=LeadStatus.choices, default=LeadStatus.NEW)
    call_status = models.CharField(max_length=50, blank=True)

    # Declarations
    DECLARATION_CHOICES = [
        ('terms', 'Terms and Conditions'),
        ('privacy', 'Privacy Policy'),
        ('risk', 'Risk Disclosure'),
        ('ib_agreement', 'IB Agreement')
    ]
    accepted_declarations = models.JSONField(default=list)  # Will store list of accepted declarations with timestamp

    # Add to Customer model:
    email_hash = models.CharField(max_length=64, unique=True, db_index=True)

    REQUIRED_FIELDS = ['first_name', 'last_name']

    class Meta:
        app_label = 'customers'
        db_table = 'customers'
        verbose_name_plural = 'customers'
        permissions = [
            # Base CRUD permissions (auto-generated by Django)
            # view_customer, add_customer, change_customer, delete_customer
            
            # Customer-specific action permissions
            ("assign_customer", "Can assign customers to CRM users"),
            ("manage_customer_status", "Can manage customer status (lead/client/IB)"),
            ("block_customer", "Can block/unblock customer"),
            ("manage_customer_trading_controls", "Can manage customer trading controls"),
            ("manage_customer_transaction_controls", "Can manage customer transaction controls"),
            ("manage_customer_sensitive_data", "Can manage sensitive customer data"),
            ("verify_customer_kyc", "Can verify customer KYC documents"),
            ("manage_customer_ib_status", "Can manage customer IB status"),
            ("export_customer_data", "Can export customer data"),
            ("view_customer_sensitive_fields", "Can view customer sensitive fields"),
            ("approve_customer_changes", "Can approve pending customer changes"),
            ("set_customer_call_status", "Can set customer call status"),
        ]

    # Required for DRF authentication
    @property
    def is_authenticated(self):
        return True

    @property
    def is_active(self):
        return self.auth.is_active if hasattr(self, 'auth') else False

    def save(self, *args, **kwargs):
        # Update email_hash if email field has changed or is being set for the first time
        if self.email and (not self.pk or 
                           not self._meta.model.objects.filter(pk=self.pk).exists() or 
                           not self.email_hash):
            self.email_hash = hashlib.sha256(self.email.lower().encode()).hexdigest()
        super().save(*args, **kwargs)

class CustomerSettings(BaseModel):
    customer = models.OneToOneField('customers.Customer', on_delete=models.CASCADE, related_name='settings')
    
    # Verification Status
    verification_status = models.JSONField(default=dict)
    
    # Controls
    transaction_controls = models.JSONField(default=dict)
    trading_controls = models.JSONField(default=dict)
    notification_preferences = models.JSONField(default=dict)
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.verification_status:
            self.verification_status = {
                "email": {
                    "verified": False,
                    "verifiedAt": None,
                    "lastVerificationSentAt": None
                },
                "phone": {
                    "verified": False,
                    "verifiedAt": None,
                    "lastVerificationSentAt": None
                },
                "mobile": {
                    "verified": False,
                    "verifiedAt": None,
                    "lastVerificationSentAt": None
                }
            }
        if not self.transaction_controls:
            self.transaction_controls = {
                "deposits": {"disabled": False, "reason": None, "disabledAt": None, "disabledBy": None},
                "withdrawals": {"disabled": False, "reason": None, "disabledAt": None, "disabledBy": None},
                "internalTransfers": {"disabled": False, "reason": None, "disabledAt": None, "disabledBy": None}
            }
        if not self.trading_controls:
            self.trading_controls = {
                "trading": {"disabled": False, "reason": None, "disabledAt": None, "disabledBy": None},
                "scalping": {"enabled": False, "enabledAt": None, "enabledBy": None},
                "selfRebate": {"enabled": False, "enabledAt": None, "enabledBy": None}
            }
        if not self.notification_preferences:
            self.notification_preferences = {
                "web": {  # WebSocket notifications
                    "enabled": True,
                    "types": {
                        "ACCOUNT": True,
                        "TRANSACTION": True,
                        "SETTINGS": True,
                        "SECURITY": True,
                        "KYC": True,
                        "SUPPORT": True,
                        "REQUEST": True,
                        "CUSTOMER": True,
                        "NEWS": True,
                        "MARKETING": True
                    },
                    "levels": {
                        "INFO": True,
                        "SUCCESS": True,
                        "WARNING": True,
                        "ERROR": True
                    }
                },
                "web_push": {  # Browser push notifications
                    "enabled": True,
                    "subscriptions": [],  # List of browser push subscriptions (endpoint, keys, etc)
                    "types": {
                        "ACCOUNT": True,
                        "TRANSACTION": True,
                        "SETTINGS": True,
                        "SECURITY": True,
                        "KYC": True,
                        "SUPPORT": True,
                        "REQUEST": True,
                        "CUSTOMER": True,
                        "NEWS": True,
                        "MARKETING": True
                    },
                    "levels": {
                        "WARNING": True,
                        "ERROR": True  # By default, only send important web push notifications
                    }
                },
                "mobile_push": {  # Mobile app push notifications
                    "enabled": True,
                    "devices": [],  # List of mobile device tokens
                    "types": {
                        "ACCOUNT": True,
                        "TRANSACTION": True,
                        "SETTINGS": True,
                        "SECURITY": True,
                        "KYC": True,
                        "SUPPORT": True,
                        "REQUEST": True,
                        "CUSTOMER": True,
                        "NEWS": True,
                        "MARKETING": True
                    },
                    "levels": {
                        "WARNING": True,
                        "ERROR": True  # By default, only send important push notifications
                    }
                },
                "email": {  # Email notifications
                    "security": True,
                    "transactions": True,
                    "accounts": True,
                    "requests": True,
                    "trading": True,
                    "kyc": True,
                    "news": True,
                    "marketing": True,
                    "levels": {
                        "WARNING": True,
                        "ERROR": True  # By default, only send important emails
                    }
                }
            }

    def should_notify(self, notification_type: str, level: str, channel: str = 'web') -> bool:
        """
        Check if a notification should be sent based on customer preferences
        """
        prefs = self.notification_preferences.get(channel, {})
        if not prefs.get('enabled', True):
            return False
            
        # Check for registered devices/subscriptions
        if channel == 'mobile_push' and not prefs.get('devices'):
            return False
        elif channel == 'web_push' and not prefs.get('subscriptions'):
            return False
            
        if channel in ['web', 'web_push', 'mobile_push']:
            type_allowed = prefs.get('types', {}).get(notification_type, True)
            level_allowed = prefs.get('levels', {}).get(level, True)
            return type_allowed and level_allowed
        elif channel == 'email':
            # Email uses different categories
            type_mapping = {
                'ACCOUNT': 'accounts',
                'TRANSACTION': 'transactions',
                'SETTINGS': 'security',
                'SECURITY': 'security',
                'KYC': 'kyc'
            }
            email_category = type_mapping.get(notification_type, 'security')
            category_allowed = prefs.get(email_category, True)
            level_allowed = prefs.get('levels', {}).get(level, True)
            return category_allowed and level_allowed

        return True

    def register_device(self, device_token: str, channel: str = 'mobile_push'):
        """
        Register a mobile device for push notifications
        """
        prefs = self.notification_preferences.get(channel, {})
        devices = prefs.get('devices', [])
        if device_token not in devices:
            devices.append(device_token)
            prefs['devices'] = devices
            self.save()

    def register_web_push(self, subscription_info: dict):
        """
        Register a browser for web push notifications
        subscription_info should contain:
        {
            'endpoint': 'https://push-service.com/...',
            'keys': {
                'p256dh': 'public key',
                'auth': 'auth secret'
            }
        }
        """
        prefs = self.notification_preferences.get('web_push', {})
        subscriptions = prefs.get('subscriptions', [])
        
        # Check if subscription already exists by endpoint
        exists = any(sub['endpoint'] == subscription_info['endpoint'] for sub in subscriptions)
        if not exists:
            subscriptions.append(subscription_info)
            prefs['subscriptions'] = subscriptions
            self.save()

    def unregister_web_push(self, endpoint: str):
        """
        Unregister a browser from web push notifications
        """
        prefs = self.notification_preferences.get('web_push', {})
        subscriptions = prefs.get('subscriptions', [])
        prefs['subscriptions'] = [sub for sub in subscriptions if sub['endpoint'] != endpoint]
        self.save()

    class Meta:
        app_label = 'customers'
        db_table = 'customer_settings'

class CustomerDetail(SensitiveFieldsTrackingMixin, BaseModel):
    customer = models.OneToOneField('customers.Customer', on_delete=models.CASCADE, related_name='details')
    
    # Personal Info - Encrypted sensitive fields
    title = models.CharField(max_length=10, blank=True)
    dob = models.DateField(null=True)
    nationality = models.CharField(max_length=2, blank=True)
    city = models.CharField(max_length=100, blank=True)
    address = models.CharField(max_length=500, blank=True)
    address2 = models.CharField(max_length=500, blank=True)
    gender = models.CharField(max_length=1, blank=True)
    zip_code = models.CharField(max_length=20, blank=True)
    language = models.CharField(max_length=2, default='EN')
    
    # Define sensitive fields for tracking
    sensitive_fields = []
    requires_approval_fields = []
    
    # Referral Info
    referral = models.CharField(
        max_length=20, 
        unique=True, 
        default=generate_referral_code
    )
    register_referral = models.CharField(max_length=100, blank=True)
    register_utm_campaign = models.CharField(max_length=100, blank=True)
    register_ib_id = models.CharField(max_length=100, blank=True)
    register_agreement_id = models.CharField(max_length=100, blank=True)
    register_url = models.CharField(max_length=255, blank=True)
    register_data = models.JSONField(default=dict)
    
    # Financial & Legal
    us_citizen = models.BooleanField(default=False)
    worked_in_financial = models.BooleanField(default=False)
    worked_in_crypto = models.BooleanField(default=False)
    tax_identification_number = models.CharField(max_length=50, blank=True)
    politically_exposed = models.BooleanField(default=False)
    annual_income = models.CharField(max_length=50, blank=True)
    source_of_funds = models.CharField(max_length=50, blank=True)
    fatca = models.BooleanField(default=False)
    
    # Document Details
    doc_type = models.CharField(max_length=50, blank=True)
    document_no = models.CharField(max_length=50, blank=True)
    date_of_issue = models.DateField(null=True)
    date_of_expiry = models.DateField(null=True)
    country_of_issue = models.CharField(max_length=2, blank=True)
    
    # Professional Info
    employment_status = models.CharField(max_length=50, blank=True)
    industry = models.CharField(max_length=100, blank=True)
    profession = models.CharField(max_length=100, blank=True)
    job_title = models.CharField(max_length=100, blank=True)
    employer = models.CharField(max_length=100, blank=True)

    # IB Specific Fields
    have_site = models.CharField(max_length=255, null=True, blank=True)
    ref_other = models.CharField(max_length=255, null=True, blank=True)
    target_countries = models.JSONField(null=True, blank=True)
    get_client = models.CharField(max_length=255, null=True, blank=True)
    expected_clients = models.IntegerField(null=True, blank=True)
    
    # External IDs
    hubspot_id = models.CharField(max_length=100, blank=True)
    old_record_id = models.CharField(max_length=100, blank=True)
    sumsub_app_id = models.CharField(max_length=100, null=True, blank=True)
    sumsub_ins_id = models.CharField(max_length=100, null=True, blank=True)

    client_at = models.DateTimeField(null=True)
    ib_at = models.DateTimeField(null=True)
    lead_at = models.DateTimeField(null=True)
    class Meta:
        app_label = 'customers'
        db_table = 'customer_details'

class CustomerAuth(SensitiveFieldsTrackingMixin, BaseModel):
    customer = models.OneToOneField('customers.Customer', on_delete=models.CASCADE, related_name='auth')
    password = models.CharField(max_length=128)
    last_login = models.DateTimeField(null=True)
    is_active = models.BooleanField(default=True)
    two_factor_enabled = models.BooleanField(default=False)
    two_factor_secret = models.CharField(max_length=32, blank=True)
    two_factor_backup_codes = models.JSONField(default=list)
    webauthn_credentials = models.JSONField(default=list)
    temp_webauthn_challenge = models.CharField(max_length=512, blank=True, null=True)
    webauthn_challenge = models.CharField(max_length=512, blank=True, null=True)
    
    # Password reset fields
    reset_token = models.CharField(max_length=255, blank=True, null=True)
    reset_token_expiry = models.DateTimeField(null=True, blank=True)
    
    # Blocking fields
    blocked = models.BooleanField(default=False)
    blocked_reason = models.TextField(blank=True, null=True)
    blocked_at = models.DateTimeField(null=True, blank=True)
    unblocked_at = models.DateTimeField(null=True, blank=True)
    blocked_by = models.ForeignKey('users.CRMUser', null=True, blank=True, on_delete=models.SET_NULL, related_name='blocked_customers')
    unblocked_by = models.ForeignKey('users.CRMUser', null=True, blank=True, on_delete=models.SET_NULL, related_name='unblocked_customers')
    
    def set_password(self, raw_password):
        self.password = make_password(raw_password)
        self.save()
    
    def check_password(self, raw_password):
        print(f"Checking password: {raw_password} {self.password}")
        return check_password(raw_password, self.password)
    
    class Meta:
        app_label = 'customers'
        db_table = 'customer_auth'



from django.apps import AppConfig

class TransactionsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'shared_models.transactions'
    verbose_name = 'Transactions'

    def ready(self):
        try:
            import shared_models.transactions.signals  # noqa
        except ImportError:
            pass 


from django.db import models
from django.utils import timezone

class BaseModel(models.Model):
    """
    Abstract base model that provides common fields for all models
    """
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.CharField(max_length=255, null=True, blank=True)
    updated_by = models.CharField(max_length=255, null=True, blank=True)

    class Meta:
        abstract = True
        ordering = ['-created_at']

    def save(self, *args, **kwargs):
        if not self.created_at:
            self.created_at = timezone.now()
        self.updated_at = timezone.now()
        super().save(*args, **kwargs) 


# Generated by Django 4.2.19 on 2025-04-11 12:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('system_settings', '0012_alter_emailcontent_content_sections'),
    ]

    operations = [
        migrations.CreateModel(
            name='KycRequirementRule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('transaction_type', models.CharField(blank=True, help_text='Apply rule only to this transaction type (e.g., DEPOSIT, WITHDRAWAL) (optional)', max_length=50, null=True)),
                ('requires_level_one', models.BooleanField(default=False, help_text='Does this rule require at least KYC Level One?')),
                ('requires_level_two', models.BooleanField(default=False, help_text='Does this rule require at least KYC Level Two?')),
                ('requires_level_three', models.BooleanField(default=False, help_text='Does this rule require at least KYC Level Three?')),
                ('payment_gateway', models.ForeignKey(blank=True, help_text='Apply rule only to this payment gateway (optional)', null=True, on_delete=django.db.models.deletion.CASCADE, to='system_settings.paymentgatewayconfig')),
                ('payment_method', models.ForeignKey(blank=True, help_text='Apply rule only to this payment method (optional)', null=True, on_delete=django.db.models.deletion.CASCADE, to='system_settings.paymentmethodconfig')),
            ],
            options={
                'verbose_name': 'KYC Requirement Rule',
                'verbose_name_plural': 'KYC Requirement Rules',
                'db_table': 'system_settings_kycrequirementrule',
                'ordering': ['payment_method', 'payment_gateway', 'transaction_type'],
            },
        ),
    ]



# Generated by Django 4.2.19 on 2025-02-21 18:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('customers', '0002_alter_customer_email_alter_customer_first_name_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='customerauth',
            name='two_factor_backup_codes',
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name='customerauth',
            name='two_factor_enabled',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='customerauth',
            name='two_factor_secret',
            field=models.CharField(blank=True, max_length=32),
        ),
        migrations.AddField(
            model_name='customerauth',
            name='webauthn_credentials',
            field=models.JSONField(default=list),
        ),
    ]



# Generated by Django 4.2.21 on 2025-06-26 13:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ib_commission', '0002_alter_commissiondistribution_processed_time'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='clientibmapping',
            index=models.Index(fields=['mt5_login', 'customer'], name='idx_client_mapping_lookup'),
        ),
        migrations.AddIndex(
            model_name='clientibmapping',
            index=models.Index(fields=['customer', 'direct_ib_customer'], name='idx_client_mapping_ib'),
        ),
        migrations.AddIndex(
            model_name='ibagreementmember',
            index=models.Index(fields=['customer', 'is_active'], name='idx_agreement_member_lookup'),
        ),
        migrations.AddIndex(
            model_name='ibagreementmember',
            index=models.Index(fields=['agreement', 'is_active'], name='idx_agreement_member_reverse'),
        ),
        migrations.AddIndex(
            model_name='ibcommissionrule',
            index=models.Index(fields=['agreement', 'commission_type', 'symbol'], name='idx_rule_lookup'),
        ),
        migrations.AddIndex(
            model_name='ibcommissionrule',
            index=models.Index(fields=['agreement', 'commission_type', 'priority'], name='idx_rule_priority'),
        ),
        migrations.AddIndex(
            model_name='ibcommissionrule',
            index=models.Index(fields=['agreement', 'account_type'], name='idx_rule_account_type'),
        ),
        migrations.AddIndex(
            model_name='ibhierarchy',
            index=models.Index(fields=['customer', 'is_active'], name='idx_hierarchy_lookup'),
        ),
        migrations.AddIndex(
            model_name='ibhierarchy',
            index=models.Index(fields=['path'], name='idx_hierarchy_path'),
        ),
    ]



# Generated by Django 4.2.21 on 2025-06-21 14:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system_settings', '0017_brokerconfiguration'),
    ]

    operations = [
        migrations.AddField(
            model_name='accounttype',
            name='demo_auto_deposit_amount',
            field=models.DecimalField(decimal_places=2, default=10000.0, help_text='Auto deposit amount for demo accounts in USD', max_digits=10),
        ),
    ]



from django.db import models
# Attempting to import from one level up, assuming common/base.py exists there
from ..common.base import BaseModel 

class DashboardTip(BaseModel):
    """Model to store actionable tips for the dashboard."""
    text = models.TextField(help_text="The content of the tip.")
    icon = models.CharField(max_length=50, blank=True, help_text="Optional FontAwesome icon class (e.g., 'faLightbulb').")
    link = models.URLField(blank=True, null=True, help_text="Optional URL for the user to click for more info.")
    is_active = models.BooleanField(default=True, help_text="Whether the tip should be displayed.")

    class Meta:
        app_label = 'ui_elements'
        db_table = 'ui_dashboard_tips'
        verbose_name = 'Dashboard Tip'
        verbose_name_plural = 'Dashboard Tips'
        ordering = ['-created_at']

    def __str__(self):
        return self.text[:50] + '...' if len(self.text) > 50 else self.text

class MotivationalQuote(BaseModel):
    """Model to store motivational quotes for the dashboard."""
    text = models.TextField(help_text="The content of the quote.")
    author = models.CharField(max_length=100, blank=True, help_text="The author of the quote.")
    link = models.URLField(blank=True, null=True, help_text="Optional URL related to the quote or author.")
    is_active = models.BooleanField(default=True, help_text="Whether the quote should be displayed.")

    class Meta:
        app_label = 'ui_elements'
        db_table = 'ui_motivational_quotes'
        verbose_name = 'Motivational Quote'
        verbose_name_plural = 'Motivational Quotes'
        ordering = ['-created_at']

    def __str__(self):
        base = self.text[:50] + '...' if len(self.text) > 50 else self.text
        if self.author:
            return f'"{base}" - {self.author}'
        return f'"{base}"' 


# Generated by Django 4.2.20 on 2025-04-19 11:09

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CustomerCryptoWallet',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('id', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('customer_id', models.IntegerField(db_index=True)),
                ('address', models.CharField(max_length=255)),
                ('network', models.CharField(max_length=255)),
                ('token', models.CharField(max_length=255)),
                ('is_primary', models.BooleanField(default=False)),
                ('is_active', models.BooleanField(default=True)),
                ('comment', models.TextField(blank=True, help_text='Additional notes about this crypto wallet', null=True)),
            ],
            options={
                'verbose_name': 'Customer Crypto Wallet',
                'verbose_name_plural': 'Customer Crypto Wallets',
                'db_table': 'shared_customer_crypto_wallets',
                'ordering': ['-created_at'],
                'permissions': [('view_customer_crypto_wallet', 'Can view customer crypto wallet'), ('delete_customer_crypto_wallet', 'Can delete customer crypto wallet'), ('change_customer_crypto_wallet', 'Can change customer crypto wallet'), ('add_customer_crypto_wallet', 'Can add customer crypto wallet')],
                'indexes': [models.Index(fields=['customer_id'], name='shared_cust_custome_3afa4d_idx'), models.Index(fields=['address', 'network'], name='shared_cust_address_0d3c47_idx'), models.Index(fields=['token'], name='shared_cust_token_2e727a_idx')],
            },
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-22 07:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0006_alter_account_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='Deal',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('deal_ticket', models.BigIntegerField(db_index=True, unique=True)),
                ('external_id', models.CharField(blank=True, db_index=True, max_length=100, null=True)),
                ('login', models.BigIntegerField(db_index=True)),
                ('dealer', models.BigIntegerField(db_index=True)),
                ('order_ticket', models.BigIntegerField(db_index=True)),
                ('action', models.IntegerField()),
                ('entry', models.IntegerField(db_index=True)),
                ('digits', models.IntegerField()),
                ('digits_currency', models.IntegerField()),
                ('contract_size', models.DecimalField(decimal_places=10, max_digits=20)),
                ('time', models.DateTimeField(db_index=True)),
                ('time_msc', models.BigIntegerField(db_index=True)),
                ('symbol', models.CharField(db_index=True, max_length=50)),
                ('price', models.DecimalField(decimal_places=10, max_digits=20)),
                ('price_sl', models.DecimalField(decimal_places=10, max_digits=20)),
                ('price_tp', models.DecimalField(decimal_places=10, max_digits=20)),
                ('volume', models.DecimalField(decimal_places=10, max_digits=20)),
                ('volume_ext', models.DecimalField(decimal_places=15, max_digits=30)),
                ('volume_closed', models.DecimalField(decimal_places=10, max_digits=20)),
                ('volume_closed_ext', models.DecimalField(decimal_places=15, max_digits=30)),
                ('profit', models.DecimalField(decimal_places=10, max_digits=20)),
                ('profit_raw', models.DecimalField(decimal_places=10, max_digits=20)),
                ('storage', models.DecimalField(decimal_places=10, max_digits=20)),
                ('commission', models.DecimalField(decimal_places=10, max_digits=20)),
                ('fee', models.DecimalField(decimal_places=10, max_digits=20)),
                ('value', models.DecimalField(decimal_places=10, max_digits=20)),
                ('rate_profit', models.DecimalField(decimal_places=10, max_digits=20)),
                ('rate_margin', models.DecimalField(decimal_places=10, max_digits=20)),
                ('expert_id', models.BigIntegerField(blank=True, null=True)),
                ('position_id', models.BigIntegerField(db_index=True)),
                ('comment', models.CharField(blank=True, max_length=255, null=True)),
                ('price_position', models.DecimalField(decimal_places=10, max_digits=20)),
                ('tick_value', models.DecimalField(decimal_places=10, max_digits=20)),
                ('tick_size', models.DecimalField(decimal_places=10, max_digits=20)),
                ('flags', models.BigIntegerField()),
                ('reason', models.IntegerField(blank=True, db_index=True, null=True)),
                ('gateway', models.CharField(blank=True, max_length=100, null=True)),
                ('price_gateway', models.DecimalField(blank=True, decimal_places=10, max_digits=20, null=True)),
                ('market_bid', models.DecimalField(decimal_places=10, max_digits=20)),
                ('market_ask', models.DecimalField(decimal_places=10, max_digits=20)),
                ('market_last', models.DecimalField(decimal_places=10, max_digits=20)),
                ('modification_flags', models.IntegerField()),
                ('raw_data', models.JSONField(blank=True, default=dict)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('account', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='accounts.account')),
            ],
            options={
                'verbose_name': 'Deal',
                'verbose_name_plural': 'Deals',
                'db_table': 'deal',
                'indexes': [models.Index(fields=['deal_ticket'], name='deal_deal_ti_b8f6f8_idx'), models.Index(fields=['login'], name='deal_login_3738f8_idx'), models.Index(fields=['account'], name='deal_account_cd7429_idx'), models.Index(fields=['time'], name='deal_time_6d2e5e_idx'), models.Index(fields=['time_msc'], name='deal_time_ms_f033a5_idx'), models.Index(fields=['position_id'], name='deal_positio_8f0d04_idx'), models.Index(fields=['symbol'], name='deal_symbol_277615_idx'), models.Index(fields=['entry'], name='deal_entry_09c766_idx'), models.Index(fields=['reason'], name='deal_reason_a48e1d_idx')],
            },
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-24 16:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('referrals', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='referralreward',
            name='payment_gateway',
            field=models.CharField(blank=True, help_text='Specific payment gateway to simulate for deposit rewards (optional).', max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name='referralreward',
            name='payment_method',
            field=models.CharField(blank=True, help_text='Specific payment method to simulate for deposit rewards (optional).', max_length=50, null=True),
        ),
    ]



# Generated by Django 5.1.4 on 2024-12-24 10:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='crmuser',
            managers=[
            ],
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-19 08:35

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CompanyCryptoWallet',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('id', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('address', models.CharField(max_length=255)),
                ('network', models.CharField(max_length=255)),
                ('token', models.CharField(max_length=255)),
                ('show_only_in_countries', models.CharField(blank=True, help_text='Comma-separated list of country codes where this wallet should be shown', max_length=255)),
                ('excluded_countries', models.CharField(blank=True, help_text='Comma-separated list of country codes where this wallet should NOT be shown', max_length=255)),
            ],
            options={
                'verbose_name': 'Company Crypto Wallet',
                'verbose_name_plural': 'Company Crypto Wallets',
                'db_table': 'company_crypto_wallets',
                'ordering': ['-created_at'],
                'permissions': [('view_company_crypto_wallet', 'Can view company crypto wallet'), ('delete_company_crypto_wallet', 'Can delete company crypto wallet'), ('change_company_crypto_wallet', 'Can change company crypto wallet'), ('add_company_crypto_wallet', 'Can add company crypto wallet')],
                'indexes': [models.Index(fields=['address', 'network'], name='company_cry_address_f19bfa_idx'), models.Index(fields=['token'], name='company_cry_token_395570_idx')],
            },
        ),
    ]



# Generated by Django 5.1.4 on 2025-02-08 12:42

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system_settings', '0007_remove_accounttype_default_group_type_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentGatewayConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('gateway', models.CharField(help_text='Unique identifier for the gateway', max_length=50, unique=True)),
                ('display_name', models.CharField(max_length=100)),
                ('enable', models.BooleanField(default=True)),
                ('enable_crm', models.BooleanField(default=True)),
                ('enable_cp', models.BooleanField(default=False)),
                ('is_external', models.BooleanField(default=False, help_text='If True, this is an external payment provider')),
                ('enabled_countries', models.JSONField(blank=True, default=list, help_text='List of country codes where this gateway is enabled')),
                ('disabled_countries', models.JSONField(blank=True, default=list, help_text='List of country codes where this gateway is disabled')),
                ('charge_fees', models.BooleanField(default=False)),
                ('fixed_fee', models.DecimalField(decimal_places=2, default=0, max_digits=10)),
                ('percentage_fee', models.DecimalField(decimal_places=2, default=0, max_digits=5)),
                ('payment_method', models.CharField(blank=True, choices=[('WIRE', 'Wire Transfer'), ('CRYPTO', 'Cryptocurrency'), ('CARD', 'Credit/Debit Card'), ('MANUAL', 'Manual'), ('SYSTEM', 'System Generated')], max_length=10, null=True)),
                ('config', models.JSONField(blank=True, default=dict, help_text='Additional configuration specific to this gateway')),
            ],
            options={
                'verbose_name': 'Payment Gateway Configuration',
                'verbose_name_plural': 'Payment Gateway Configurations',
                'db_table': 'payment_gateway_configs',
                'ordering': ['gateway'],
            },
        ),
        migrations.CreateModel(
            name='PaymentMethodConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('payment_method', models.CharField(choices=[('WIRE', 'Wire Transfer'), ('CRYPTO', 'Cryptocurrency'), ('CARD', 'Credit/Debit Card'), ('MANUAL', 'Manual'), ('SYSTEM', 'System Generated')], max_length=10)),
                ('is_enabled', models.BooleanField(default=True)),
                ('requires_manual_approval', models.BooleanField(default=True)),
                ('requires_evidence', models.BooleanField(default=True)),
                ('display_name', models.CharField(max_length=100)),
                ('allowed_transaction_types', models.JSONField(default=list, help_text='List of transaction types allowed for this payment method')),
                ('min_amount', models.DecimalField(blank=True, decimal_places=2, max_digits=20, null=True)),
                ('max_amount', models.DecimalField(blank=True, decimal_places=2, max_digits=20, null=True)),
                ('metadata_schema', models.JSONField(default=dict, help_text='Required fields and validations for this payment method')),
                ('approval_roles', models.JSONField(default=list, help_text='Roles that can approve transactions with this payment method')),
                ('allowed_gateways', models.ManyToManyField(related_name='payment_methods', to='system_settings.paymentgatewayconfig')),
                ('default_gateway', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='default_for_methods', to='system_settings.paymentgatewayconfig')),
            ],
            options={
                'db_table': 'payment_method_configs',
                'ordering': ['payment_method'],
            },
        ),
        migrations.CreateModel(
            name='TransactionTypeConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('transaction_type', models.CharField(choices=[('DEPOSIT', 'Deposit'), ('WITHDRAWAL', 'Withdrawal'), ('BONUS', 'Bonus'), ('REBATE', 'Rebate'), ('COMMISSION', 'Commission'), ('CORRECTION', 'Correction'), ('INTERNAL_TRANSFER', 'Internal Transfer'), ('CREDIT', 'Credit'), ('CHARGE', 'Charge'), ('COMPENSATION', 'Compensation')], max_length=20)),
                ('allows_in', models.BooleanField(default=True)),
                ('allows_out', models.BooleanField(default=True)),
                ('requires_approval', models.BooleanField(default=True)),
                ('approval_roles', models.JSONField(default=list, help_text='Roles that can approve this transaction type')),
                ('min_amount', models.DecimalField(blank=True, decimal_places=2, max_digits=20, null=True)),
                ('max_amount', models.DecimalField(blank=True, decimal_places=2, max_digits=20, null=True)),
                ('rules', models.JSONField(default=dict, help_text='Type-specific configuration and rules')),
                ('requires_reason', models.BooleanField(default=False)),
                ('requires_documentation', models.BooleanField(default=False)),
                ('auto_approve_threshold', models.DecimalField(blank=True, decimal_places=2, help_text='Amount below which transactions are auto-approved', max_digits=20, null=True)),
            ],
            options={
                'db_table': 'transaction_type_configs',
                'ordering': ['transaction_type'],
                'unique_together': {('transaction_type',)},
            },
        ),
    ]



# Generated by Django 4.2.19 on 2025-02-09 17:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0004_alter_account_created_at_alter_account_created_by_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='account',
            name='currency',
            field=models.CharField(default='USD', max_length=3),
        ),
    ]



# Generated by Django 4.2.19 on 2025-02-26 14:04

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('customers', '0006_customerauth_webauthn_challenge'),
    ]

    operations = [
        migrations.AddField(
            model_name='customerauth',
            name='blocked',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='customerauth',
            name='blocked_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='customerauth',
            name='blocked_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='blocked_customers', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='customerauth',
            name='blocked_reason',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='customerauth',
            name='unblocked_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='customerauth',
            name='unblocked_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='unblocked_customers', to=settings.AUTH_USER_MODEL),
        ),
    ]



"""
Models for the IB (Introducing Broker) Commission System.

This module contains models for managing IB hierarchies, agreements, commission rules,
and tracking of commission distribution as detailed in the IB Commission Implementation Guide.
"""
import logging
from django.db import models
from django.utils.translation import gettext_lazy as _
from shared_models.common.base import BaseModel
from shared_models.trading.models import Deal


logger = logging.getLogger(__name__)


class IBHierarchy(BaseModel):
    """
    Tracks the hierarchical relationships between IBs (Introducing Brokers).
    
    This model maintains the parent-child relationships between IBs,
    allowing for multi-level IB structures and commission distribution.
    """
    customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, related_name='ib_hierarchy')
    parent_customer = models.ForeignKey('customers.Customer', on_delete=models.SET_NULL, null=True, 
                                       blank=True, related_name='child_ibs')
    level = models.IntegerField(help_text='0 for master IB, increments for each level down')
    path = models.TextField(help_text='Stored as dot-separated customer ids, e.g., "1001.1002.1003"')
    
    # Add account reference while keeping mt5_login for backward compatibility
    ib_account = models.ForeignKey('accounts.Account', on_delete=models.PROTECT, 
                                 related_name='ib_hierarchy_entry', null=True, blank=True,
                                 help_text='IB account that receives commissions')
    mt5_login = models.IntegerField(help_text='MT5 login for deal processing')
    
    is_active = models.BooleanField(default=True)

    def save(self, *args, **kwargs):
        """
        Override save to keep ib_account and mt5_login in sync.
        If ib_account is provided, update mt5_login to match.
        """
        if self.ib_account and self.ib_account.login != self.mt5_login:
            self.mt5_login = self.ib_account.login
        super().save(*args, **kwargs)

    class Meta:
        app_label = 'ib_commission'
        db_table = 'ib_hierarchy'
        verbose_name = 'IB Hierarchy'
        verbose_name_plural = 'IB Hierarchies'
        indexes = [
            models.Index(fields=['mt5_login']),
            models.Index(fields=['path']),
            models.Index(fields=['customer', 'is_active'], name='idx_hierarchy_lookup'),
            models.Index(fields=['path'], name='idx_hierarchy_path'),  # For path__contains queries
        ]
        constraints = [
            models.UniqueConstraint(fields=['customer'], name='unique_customer_in_hierarchy')
        ]


class IBAgreement(BaseModel):
    """
    Represents an IB commission agreement.
    
    This is the base agreement model that contains general information about the agreement.
    Specific commission rules are defined in the IBCommissionRule model.
    """
    name = models.CharField(max_length=100)
    description = models.TextField(blank=True)
    is_active = models.BooleanField(default=True)

    class Meta:
        app_label = 'ib_commission'
        db_table = 'ib_agreements'
        verbose_name = 'IB Agreement'
        verbose_name_plural = 'IB Agreements'

class ClientIBMapping(BaseModel):
    """
    Maps clients to their IBs in the hierarchy.
    
    This model maintains the relationship between clients and their IBs,
    including information about the direct IB and the master IB.
    """
    customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, 
                               related_name='client_ib_mappings')
    direct_ib_customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, 
                                        related_name='direct_clients')
    master_ib_customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, 
                                        related_name='all_hierarchy_clients')
    agreement = models.ForeignKey(IBAgreement, on_delete=models.SET_NULL, null=True, blank=True,
                               related_name='client_mappings', 
                               help_text='Default agreement for this client-IB relationship')
    ib_path = models.TextField(help_text='Full path of IBs using customer ids')
    mt5_login = models.IntegerField(help_text="Client's MT5 login", null=True, blank=True)

    class Meta:
        app_label = 'ib_commission'
        db_table = 'client_ib_mapping'
        verbose_name = 'Client IB Mapping'
        verbose_name_plural = 'Client IB Mappings'
        indexes = [
            models.Index(fields=['mt5_login']),
            models.Index(fields=['customer']),
            models.Index(fields=['mt5_login', 'customer'], name='idx_client_mapping_lookup'),
            models.Index(fields=['customer', 'direct_ib_customer'], name='idx_client_mapping_ib'),
        ]


class IBAgreementMember(BaseModel):
    """
    Links customers (IBs) to agreements.
    
    This model associates IBs with specific agreements and tracks
    when the agreement is active for that IB.
    """
    agreement = models.ForeignKey(IBAgreement, on_delete=models.CASCADE, related_name='members')
    customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, 
                               related_name='ib_agreements')
    is_self_rebate = models.BooleanField(default=False, 
                                       help_text='Whether the IB can receive rebates from their own trading')
    start_date = models.DateTimeField(null=True, blank=True)
    end_date = models.DateTimeField(null=True, blank=True)
    is_active = models.BooleanField(default=True)

    class Meta:
        app_label = 'ib_commission'
        db_table = 'ib_agreement_members'
        verbose_name = 'IB Agreement Member'
        verbose_name_plural = 'IB Agreement Members'
        indexes = [
            models.Index(fields=['customer', 'is_active'], name='idx_agreement_member_lookup'),
            models.Index(fields=['agreement', 'is_active'], name='idx_agreement_member_reverse'),
        ]


class IBAccountAgreement(BaseModel):
    """
    Account-specific agreement assignments.
    
    This model allows for assigning specific agreements to individual
    trading accounts, overriding the default agreement for a client.
    """
    agreement = models.ForeignKey(IBAgreement, on_delete=models.PROTECT, related_name='account_agreements')
    customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, 
                               related_name='account_ib_agreements')
    
    # Add account reference while keeping mt5_login for backward compatibility
    account = models.ForeignKey('accounts.Account', on_delete=models.PROTECT,
                             related_name='ib_agreement_overrides', null=True, blank=True,
                             help_text='Client account with agreement override')
    mt5_login = models.IntegerField(help_text='Specific MT5 account login')
    
    ib_customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, 
                                  related_name='ib_account_agreements')
    is_self_rebate = models.BooleanField(default=False)
    start_date = models.DateTimeField()
    end_date = models.DateTimeField(null=True, blank=True)
    is_active = models.BooleanField(default=True)

    def save(self, *args, **kwargs):
        """
        Override save to keep account and mt5_login in sync.
        If account is provided, update mt5_login to match.
        """
        if self.account and self.account.mt5_login != self.mt5_login:
            self.mt5_login = self.account.mt5_login
        super().save(*args, **kwargs)

    class Meta:
        app_label = 'ib_commission'
        db_table = 'ib_account_agreements'
        verbose_name = 'IB Account Agreement'
        verbose_name_plural = 'IB Account Agreements'
        indexes = [
            models.Index(fields=['mt5_login']),
            models.Index(fields=['customer']),
        ]
        constraints = [
            models.UniqueConstraint(fields=['mt5_login', 'ib_customer'], name='unique_account_ib_agreement')
        ]


class IBCommissionRule(BaseModel):
    """
    Defines rules for commission calculations.
    
    This model contains the rules for calculating commissions based on
    various factors such as account type, symbol, and calculation method.
    """
    CALCULATION_TYPES = [
        ('LOT_BASED', 'Lot Based'),
        ('PERCENTAGE', 'Percentage'),
        ('PIP_VALUE', 'Pip Value'),
        ('TIERED', 'Tiered Volume'),
    ]
    
    COMMISSION_TYPES = [
        ('REBATE', 'Rebate'),  # Broker pays IB
        ('COMMISSION', 'Commission'),  # Client pays IB
    ]
    
    agreement = models.ForeignKey(IBAgreement, on_delete=models.CASCADE, related_name='commission_rules')
    commission_type = models.CharField(max_length=10, choices=COMMISSION_TYPES, 
                                    help_text='Whether this rule applies to rebates or commissions')
    account_type = models.ForeignKey(
        'system_settings.AccountType',
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name='commission_rules',
        help_text='NULL means all account types'
    )
    symbol = models.CharField(max_length=50, null=True, blank=True, 
                            help_text='NULL means all symbols')
    calculation_type = models.CharField(max_length=20, choices=CALCULATION_TYPES)
    value = models.DecimalField(max_digits=20, decimal_places=5, 
                              help_text='Amount per lot/percentage/pip')
    lot_size = models.DecimalField(max_digits=10, decimal_places=2, default=1.00,
                                 help_text='Number of lots this value applies to (for LOT_BASED calculation)')
    min_volume = models.DecimalField(max_digits=10, decimal_places=2, default=0.00,
                                   help_text='Minimum volume (in lots) required to qualify for this rule')
    keep_percentage = models.DecimalField(max_digits=5, decimal_places=2, default=100.00,
                                      help_text='Percentage of commission to keep (0-100)')
    pass_up_percentage = models.DecimalField(max_digits=5, decimal_places=2, default=0.00,
                                         help_text='Percentage of commission to pass up to parent IB (0-100)')
    min_amount = models.DecimalField(max_digits=20, decimal_places=5, 
                                   help_text='Minimum commission amount')
    max_amount = models.DecimalField(max_digits=20, decimal_places=5, 
                                   help_text='Maximum commission amount')
    min_trade_time = models.IntegerField(default=0, 
                                       help_text='Minimum trade duration in seconds (anti-scalping)')
    conditions = models.JSONField(default=dict, blank=True, 
                                help_text='Additional conditions (volume ranges, etc)')
    priority = models.IntegerField(default=10, 
                                 help_text='Rule priority (lower number = higher priority)')

    class Meta:
        app_label = 'ib_commission'
        db_table = 'ib_commission_rules'
        verbose_name = 'IB Commission Rule'
        verbose_name_plural = 'IB Commission Rules'
        indexes = [
            models.Index(fields=['agreement', 'priority']),
            models.Index(fields=['account_type', 'symbol']),
            models.Index(fields=['commission_type']),
            models.Index(fields=['agreement', 'commission_type', 'symbol'], name='idx_rule_lookup'),
            models.Index(fields=['agreement', 'commission_type', 'priority'], name='idx_rule_priority'),
            models.Index(fields=['agreement', 'account_type'], name='idx_rule_account_type'),
        ]
        constraints = [
            models.UniqueConstraint(
                fields=['agreement', 'commission_type', 'account_type', 'symbol', 'calculation_type'],
                name='unique_commission_rule_config'
            ),
        ]


class CommissionTracking(BaseModel):
    """
    Tracks all commission operations.
    
    This model records the details of each commission transaction,
    including the client, IB, and commission amount.
    """
    COMMISSION_TYPES = [
        ('REBATE', 'Rebate'),
        ('COMMISSION', 'Commission'),
    ]
    deal = models.OneToOneField(Deal, on_delete=models.CASCADE, related_name='commission_tracking')
    customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, related_name='commissions_generated')
    direct_ib_customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, related_name='direct_commissions')
    commission_type = models.CharField(max_length=10, choices=COMMISSION_TYPES)
    rule = models.ForeignKey(IBCommissionRule, on_delete=models.SET_NULL, null=True, blank=True, related_name='commissions')
    amount = models.DecimalField(max_digits=20, decimal_places=5)
    processed_time = models.DateTimeField(null=True, blank=True)

    class Meta:
        app_label = 'ib_commission'
        db_table = 'commission_tracking'
        verbose_name = 'Commission Tracking'
        verbose_name_plural = 'Commission Tracking'
        indexes = [
            models.Index(fields=['customer']),
            models.Index(fields=['direct_ib_customer']),
            models.Index(fields=['rule']),
            models.Index(fields=['processed_time']),
        ]


class CommissionDistribution(BaseModel):
    """
    Records commission distributions to IBs.
    
    This model tracks how commissions are distributed through the IB hierarchy,
    recording the amount each IB receives from a transaction.
    """
    DISTRIBUTION_TYPES = [
        ('REBATE', 'Rebate'),
        ('COMMISSION', 'Commission'),
    ]

    PROCESSING_STATUS_CHOICES = [
        ('PENDING', 'Pending'),
        ('PROCESSED', 'Processed'),
        ('FAILED', 'Failed'),
        ('CANCELLED', 'Cancelled'),
        ('DELAYED', 'Delayed - Pending Open Deal'),
        ('SKIPPED', 'Skipped'),
    ]

    DELAYED_REASON_CHOICES = [
        ('MISSING_OPEN_DEAL', 'Missing Opening Deal'),
    ]
    
    commission_tracking = models.ForeignKey(
        CommissionTracking,
        on_delete=models.PROTECT, # Or CASCADE? Consider lifecycle. PROTECT is safer initially.
        related_name='distributions',
        db_index=True # Explicitly add index if not added automatically by PROTECT/FK
    )
    customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, related_name='commission_distributions')
    ib_account = models.ForeignKey('accounts.Account', on_delete=models.PROTECT, related_name='received_distributions', null=True, blank=True, help_text="IB's account receiving commissions")
    mt5_login = models.IntegerField(help_text="IB's MT5 login")
    amount = models.DecimalField(max_digits=20, decimal_places=5)
    level = models.IntegerField(help_text='IB level in the hierarchy')
    distribution_type = models.CharField(max_length=10, choices=DISTRIBUTION_TYPES)
    is_pass_up = models.BooleanField(default=False, help_text='Whether this distribution is passed up to parent IB')
    processed_time = models.DateTimeField(null=True, blank=True)
    is_processed = models.BooleanField(default=False, help_text='Whether this distribution has been processed by the financial system')
    processing_status = models.CharField(max_length=20, choices=PROCESSING_STATUS_CHOICES, default='PENDING', help_text='Current status of the distribution processing')
    processing_notes = models.TextField(blank=True, null=True, help_text='Additional notes about processing status')
    transaction = models.ForeignKey('transactions.CommissionRebateTransaction', on_delete=models.SET_NULL, related_name='distributions', null=True, blank=True, help_text='Associated financial transaction record')
    rule = models.ForeignKey(IBCommissionRule, on_delete=models.SET_NULL, null=True, blank=True, related_name='distributions')
    retry_count = models.IntegerField(default=0, help_text='Number of times processing has been retried for delayed distributions')
    delayed_reason = models.CharField(max_length=50, null=True, blank=True, choices=DELAYED_REASON_CHOICES, help_text='Reason why processing is delayed')

    def save(self, *args, **kwargs):
        """
        Override save to keep ib_account and mt5_login in sync.
        If ib_account is provided, update mt5_login to match.
        """
        if self.ib_account and self.ib_account.login != self.mt5_login:
            self.mt5_login = self.ib_account.login
        super().save(*args, **kwargs)

    class Meta:
        app_label = 'ib_commission'
        db_table = 'commission_distribution'
        verbose_name = 'Commission Distribution'
        verbose_name_plural = 'Commission Distributions'
        indexes = [
            models.Index(fields=['mt5_login']),
            models.Index(fields=['customer']),
            models.Index(fields=['processed_time']),
            models.Index(fields=['is_processed']),
            models.Index(fields=['processing_status']),
            models.Index(fields=['commission_tracking']),
        ]
        constraints = [
            models.UniqueConstraint(
                fields=['commission_tracking', 'customer', 'rule', 'is_pass_up'], 
                name='unique_distribution_per_ib_rule_type'
            )
        ]


class IBViewPermission(BaseModel):
    """
    Controls hierarchy viewing permissions for IBs.
    
    This model defines which IBs can view the full hierarchy
    and which are restricted to their own sub-hierarchy.
    """
    customer = models.ForeignKey('customers.Customer', on_delete=models.PROTECT, 
                               related_name='ib_view_permissions')
    can_view_full_hierarchy = models.BooleanField(default=False)
    created_by = models.ForeignKey('users.CRMUser', on_delete=models.SET_NULL, 
                                 null=True, related_name='created_ib_permissions')

    class Meta:
        app_label = 'ib_commission'
        db_table = 'ib_view_permissions'
        verbose_name = 'IB View Permission'
        verbose_name_plural = 'IB View Permissions'


# Import the onboarding model
from .model.ib_onboarding import IBOnboardingStatus
from .model.ib_questionnaire import IBQuestionnaire


from django.db import models
from django.core.validators import validate_email
from django.core.exceptions import ValidationError
from ...constants import EmailProvider, SettingType, ContentType, BaseEmailVariables, SystemEmailTrigger
from .base import SystemSetting
import re

class EmailConfiguration(SystemSetting):
    """
    Email provider configuration for the system.
    Credentials are stored in Azure Key Vault.
    """
    provider = models.CharField(
        max_length=50,
        choices=[(p.value, p.value) for p in EmailProvider],
        default=EmailProvider.SMTP
    )
    from_email = models.EmailField(validators=[validate_email])
    reply_to = models.EmailField(validators=[validate_email], null=True, blank=True)
    
    # SMTP specific settings
    use_tls = models.BooleanField(
        default=True,
        help_text="Use TLS encryption for SMTP"
    )
    use_ssl = models.BooleanField(
        default=False,
        help_text="Use SSL encryption for SMTP. Don't enable both TLS and SSL."
    )
    timeout = models.IntegerField(
        default=30,
        help_text="Connection timeout in seconds"
    )
    debug_level = models.IntegerField(
        default=0,
        help_text="SMTP debug level (0-2). Higher values for more verbose logging."
    )

    def clean(self):
        """Validate the model"""
        super().clean()
        
        if self.use_tls and self.use_ssl:
            raise ValidationError({
                'use_ssl': "Cannot use both TLS and SSL simultaneously",
                'use_tls': "Cannot use both TLS and SSL simultaneously"
            })

    def save(self, *args, **kwargs):
        self.clean()
        self.setting_type = SettingType.EMAIL.value
        super().save(*args, **kwargs)

    class Meta:
        db_table = 'system_email_configuration'
        app_label = 'system_settings'
        verbose_name = 'Email Configuration'
        verbose_name_plural = 'Email Configurations'

    def __str__(self):
        return f"{self.name} ({self.provider})"

class EmailTemplate(SystemSetting):
    """
    HTML email template with placeholders for content sections.
    Example:
    <div>___header___</div>
    <div>___content1___</div>
    <button>___button1Title___</button>
    """
    html_template = models.TextField(
        help_text="HTML template with placeholders like ___content1___, ___button1Title___",
        default="""
<!DOCTYPE html>
<html>
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
</head>
<body>
    <div>___header___</div>
    <div>___main_content___</div>
    <div>___footer___</div>
</body>
</html>
        """.strip()
    )
    content_placeholders = models.JSONField(
        help_text="List of content placeholders in the template (e.g., ['content1', 'button1Title'])",
        default=list
    )

    def save(self, *args, **kwargs):
        # Extract placeholders from html_template using regex
        pattern = r'___([a-zA-Z][a-zA-Z0-9_]*)___'
        placeholders = re.findall(pattern, self.html_template)
        
        # Handle the case where html_template might have extra whitespace
        if not placeholders:
            # Try with stripped template
            placeholders = re.findall(pattern, self.html_template.strip())
        
        self.content_placeholders = list(set(placeholders))  # Remove duplicates
        self.setting_type = SettingType.EMAIL.value
        super().save(*args, **kwargs)

    class Meta:
        db_table = 'system_email_template'
        app_label = 'system_settings'
        verbose_name = 'Email Template'
        verbose_name_plural = 'Email Templates'

class SystemEmail(SystemSetting):
    """
    System email configuration that defines when and how emails are sent.
    Links a template with its content and trigger events.
    """
    template = models.ForeignKey(EmailTemplate, on_delete=models.PROTECT)
    available_variables = models.JSONField(
        help_text="Additional variables specific to this type of system email (e.g., ['order_number', 'amount']). Base variables are automatically included.",
        default=list
    )
    cc = models.JSONField(default=list)  # List of email addresses
    bcc = models.JSONField(default=list)  # List of email addresses
    trigger_event = models.CharField(
        max_length=255,
        help_text="Event that triggers this email. Must be one of the predefined trigger events."
    )

    @property
    def all_available_variables(self) -> list:
        """Get all available variables including base variables"""
        return BaseEmailVariables.get_all_variables() + self.available_variables

    def clean(self):
        """Validate the model"""
        super().clean()
        
        # Validate trigger event
        valid_triggers = SystemEmailTrigger.get_all_triggers()
        if self.trigger_event not in valid_triggers:
            raise ValidationError({
                'trigger_event': f'Invalid trigger event. Must be one of: {", ".join(valid_triggers)}'
            })

        # Get required variables for this trigger
        required_vars = SystemEmailTrigger.get_trigger_variables().get(self.trigger_event, [])
        # Check if all required variables are available
        available_vars = set(self.all_available_variables)
        missing_vars = set(required_vars) - available_vars
        if missing_vars:
            raise ValidationError({
                'available_variables': f'Missing required variables for trigger {self.trigger_event}: {", ".join(missing_vars)}'
            })
        
        # Validate CC and BCC email addresses
        for email in self.cc + self.bcc:
            try:
                validate_email(email)
            except ValidationError:
                raise ValidationError({
                    'cc' if email in self.cc else 'bcc': f'Invalid email address: {email}'
                })

    def save(self, *args, **kwargs):
        self.clean()
        self.setting_type = SettingType.EMAIL.value
        super().save(*args, **kwargs)

    class Meta:
        db_table = 'system_email'
        app_label = 'system_settings'
        verbose_name = 'System Email'
        verbose_name_plural = 'System Emails'

class EmailContent(SystemSetting):
    """
    Multi-language content for system emails.
    Content sections that will be inserted into template placeholders.
    Can use variables like {{ user_name }} which will be replaced at send time.
    Variables are not restricted to predefined ones - any variable in the context_data will be replaced.
    """
    system_email = models.ForeignKey(SystemEmail, on_delete=models.CASCADE, related_name='contents')
    language_code = models.CharField(max_length=10)  # e.g., 'en', 'es', 'fr'
    subject = models.CharField(max_length=255)  # Email subject line
    content_sections = models.JSONField(
        help_text="Dict mapping template placeholders to content structures. Each structure has 'default' content and optional 'condition_sets'. Content can include {{ variables }}.",
        default=dict  # Default remains dict, but structure inside will change
    )
    content_type = models.CharField(
        max_length=20,
        choices=[(t.value, t.value) for t in ContentType],
        default=ContentType.HTML
    )

    def get_used_variables(self) -> set:
        """Extract all variables used in subject and content sections"""
        pattern = r'{{\s*([a-zA-Z_][a-zA-Z0-9_]*)\s*}}'
        variables = set()
        
        # Check subject
        variables.update(re.findall(pattern, self.subject))
        
        # Check all content sections
        for content in self.content_sections.values():
            variables.update(re.findall(pattern, content))
            
        return variables

    def clean(self):
        """Validate the model"""
        super().clean()

        template_placeholders = set(self.system_email.template.content_placeholders)
        content_sections_keys = set(self.content_sections.keys())

        # Check for missing required placeholders
        missing_placeholders = template_placeholders - content_sections_keys
        if missing_placeholders:
            raise ValidationError({
                'content_sections': f'Missing content structure for required placeholders: {", ".join(missing_placeholders)}'
            })

        # Check for extra placeholders that aren't in the template
        extra_placeholders = content_sections_keys - template_placeholders
        if extra_placeholders:
            raise ValidationError({
                'content_sections': f'Content sections contain placeholders not in template: {", ".join(extra_placeholders)}'
            })

        # Validate the structure of each content section
        for placeholder, section_data in self.content_sections.items():
            if not isinstance(section_data, dict):
                raise ValidationError({
                    'content_sections': f"Structure for placeholder '{placeholder}' must be a dictionary."
                })
            if 'default' not in section_data:
                 raise ValidationError({
                    'content_sections': f"Structure for placeholder '{placeholder}' must contain a 'default' key."
                })
            if 'condition_sets' in section_data:
                if not isinstance(section_data['condition_sets'], list):
                     raise ValidationError({
                        'content_sections': f"'condition_sets' for placeholder '{placeholder}' must be a list."
                    })
                for i, condition_set in enumerate(section_data['condition_sets']):
                    if not isinstance(condition_set, dict):
                        raise ValidationError({
                            'content_sections': f"Condition set {i+1} for placeholder '{placeholder}' must be a dictionary."
                        })
                    required_keys = {'conditions', 'content'}
                    if not required_keys.issubset(condition_set.keys()):
                        raise ValidationError({
                            'content_sections': f"Condition set {i+1} for placeholder '{placeholder}' is missing required keys: {', '.join(required_keys - set(condition_set.keys()))}."
                        })
                    if not isinstance(condition_set['conditions'], list):
                         raise ValidationError({
                            'content_sections': f"'conditions' in set {i+1} for placeholder '{placeholder}' must be a list."
                        })
                    # Basic validation for individual conditions can be added here if needed
                    # e.g., check if each condition is a dict with a 'type' key

        # REMOVED: Variable availability check - too complex with conditional logic
        # used_variables = self.get_used_variables() # This method might need adjustment too if we want to check variables within conditions
        # available_variables = set(self.system_email.all_available_variables)
        # missing_variables = used_variables - available_variables
        # if missing_variables:
        #     raise ValidationError({
        #         'content_sections': f'Content uses variables that are not available: {", ".join(missing_variables)}'
        #     })

    def save(self, *args, **kwargs):
        # Clean is called automatically by Django's save process if not disabled
        self.setting_type = SettingType.EMAIL.value
        super().save(*args, **kwargs)

    class Meta:
        db_table = 'system_email_content'
        app_label = 'system_settings'
        verbose_name = 'Email Content'
        verbose_name_plural = 'Email Contents'
        unique_together = ('system_email', 'language_code', 'content_type')



# Generated by Django 4.2.19 on 2025-02-08 19:05

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system_settings', '0009_alter_accountsequence_created_at_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentgatewayconfig',
            name='gateway',
            field=models.CharField(help_text='Unique identifier for the gateway. Must be uppercase with underscores (e.g. NEG_BALANCE_COVERAGE)', max_length=50, unique=True, validators=[django.core.validators.RegexValidator(message='Gateway name must be uppercase and contain only letters, numbers, and underscores', regex='^[A-Z][A-Z0-9_]*$')]),
        ),
        migrations.AlterField(
            model_name='paymentmethodconfig',
            name='allowed_gateways',
            field=models.ManyToManyField(blank=True, related_name='payment_methods', to='system_settings.paymentgatewayconfig'),
        ),
    ]



from django.db import models
from django.utils import timezone
from shared_models.common.base import BaseModel


class IBQuestionnaire(BaseModel):
    """
    IB business information questionnaire - unified for all flows.
    Can be created from CP (with IBRequest) or CRM (standalone).
    """
    customer = models.OneToOneField(
        'customers.Customer', 
        on_delete=models.CASCADE,
        related_name='ib_questionnaire'
    )
    
    # Business Information
    target_countries = models.JSONField(
        help_text="List of country codes the IB plans to target"
    )
    expected_clients = models.IntegerField(
        help_text="Expected number of clients per month"
    )
    have_site = models.CharField(
        max_length=255,
        help_text="Whether IB has a website/platform"
    )
    get_client = models.TextField(
        help_text="How the IB plans to acquire clients"
    )
    ref_other = models.TextField(
        blank=True,
        help_text="Previous experience or other referral information"
    )
    
    # Metadata
    source = models.CharField(
        max_length=10, 
        choices=[
            ('CP', 'Client Portal'),
            ('CRM', 'CRM')
        ],
        help_text="Where the questionnaire was submitted from"
    )
    submitted_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    submitted_by = models.ForeignKey(
        'users.CRMUser', 
        null=True, 
        blank=True,
        on_delete=models.SET_NULL,
        help_text="CRM user who submitted on behalf of IB (if source=CRM)"
    )
    
    class Meta:
        db_table = 'ib_questionnaire'
        app_label = 'ib_commission'
        verbose_name = 'IB Questionnaire'
        verbose_name_plural = 'IB Questionnaires'
        
    def __str__(self):
        return f"IB Questionnaire for {self.customer.email}"


from django.apps import AppConfig


class CompanyUpiIdConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'shared_models.upi_id'
    verbose_name = 'Company UPI IDs'
    label = 'shared_company_upi_id'



from django.apps import AppConfig

class CustomersConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'shared_models.customers'
    label = 'customers'
    verbose_name = 'Shared Customers'
    
    def ready(self):
        try:
            import shared_models.customers.signals  # noqa
        except ImportError:
            pass 


# Generated by Django 4.2.19 on 2025-03-02 21:07

import uuid
from django.db import migrations, models

def generate_unique_hashes(apps, schema_editor):
    Customer = apps.get_model('customers', 'Customer')
    for customer in Customer.objects.all():
        customer.email_hash = uuid.uuid4().hex
        customer.save(update_fields=['email_hash'])

class Migration(migrations.Migration):

    dependencies = [
        ('customers', '0007_customerauth_blocked_customerauth_blocked_at_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='email_hash',
            field=models.CharField(db_index=True, max_length=64, unique=True, null=True),
        ),
        migrations.RunPython(generate_unique_hashes),
        migrations.AlterField(
            model_name='customer',
            name='email_hash',
            field=models.CharField(db_index=True, max_length=64, unique=True),
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-10 10:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('system_settings', '0011_tradingplatformserverlog'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailcontent',
            name='content_sections',
            field=models.JSONField(default=dict, help_text="Dict mapping template placeholders to content structures. Each structure has 'default' content and optional 'condition_sets'. Content can include {{ variables }}."),
        ),
    ]



# Generated by Django 5.1.4 on 2025-01-02 12:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0005_remove_crmuser_is_2fa_enabled_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='usersettings',
            old_name='preferred_2fa_method',
            new_name='default_2fa_method',
        ),
        migrations.AddField(
            model_name='usersettings',
            name='active_2fa_methods',
            field=models.JSONField(blank=True, default=dict, null=True),
        ),
        migrations.AlterModelTable(
            name='usersettings',
            table=None,
        ),
    ]



# Generated by Django 4.2.20 on 2025-06-10 10:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('transactions', '0010_alter_transaction_payment_method'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='transaction',
            options={'ordering': ['-created_at'], 'permissions': [('approve_transaction', 'Can approve transactions'), ('reject_transaction', 'Can reject transactions')]},
        ),
    ]



# Generated by Django 4.2.20 on 2025-06-10 10:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0014_alter_crmuser_groups_alter_crmuser_user_permissions_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='crmuser',
            options={'permissions': [('change_access_crmuser', 'Can block/unblock user'), ('reset_password_crmuser', 'Can reset user passwords'), ('can_assign_users', 'Can assign users to customers'), ('can_be_assigned', 'Can be assigned to customers'), ('view_own_dashboard', 'Can view own dashboard metrics'), ('view_all_users_activity', 'Can view all users activity analytics'), ('view_team_analytics', 'Can view team analytics'), ('view_customer_analytics', 'Can view customer analytics'), ('view_transaction_analytics', 'Can view transaction analytics'), ('view_ib_analytics', 'Can view IB analytics'), ('view_support_analytics', 'Can view support analytics'), ('view_request_analytics', 'Can view request analytics')], 'verbose_name': 'user', 'verbose_name_plural': 'users'},
        ),
    ]



# Generated by Django 5.1.4 on 2025-01-15 15:11

import django.core.validators
import django.db.models.deletion
import shared_models.constants
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SystemSetting',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('name', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True, null=True)),
                ('setting_type', models.CharField(choices=[('email', 'email'), ('security', 'security'), ('kyc', 'kyc'), ('commission', 'commission')], max_length=50)),
                ('status', models.CharField(choices=[('active', 'active'), ('inactive', 'inactive'), ('pending', 'pending')], default=shared_models.constants.SettingStatus['INACTIVE'], max_length=20)),
                ('is_enabled', models.BooleanField(default=False)),
                ('metadata', models.JSONField(blank=True, default=dict)),
            ],
            options={
                'verbose_name': 'System Setting',
                'verbose_name_plural': 'System Settings',
                'db_table': 'system_settings',
                'ordering': ['-created_at'],
                'permissions': [('add_system_setting', 'Can add system setting'), ('change_system_setting', 'Can change system setting'), ('delete_system_setting', 'Can delete system setting'), ('view_system_setting', 'Can view system setting')],
            },
        ),
        migrations.CreateModel(
            name='EmailConfiguration',
            fields=[
                ('systemsetting_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='system_settings.systemsetting')),
                ('provider', models.CharField(choices=[('smtp', 'smtp'), ('sendgrid', 'sendgrid')], default=shared_models.constants.EmailProvider['SMTP'], max_length=50)),
                ('from_email', models.EmailField(max_length=254, validators=[django.core.validators.EmailValidator()])),
                ('reply_to', models.EmailField(blank=True, max_length=254, null=True, validators=[django.core.validators.EmailValidator()])),
                ('use_tls', models.BooleanField(default=True, help_text='Use TLS encryption for SMTP')),
                ('use_ssl', models.BooleanField(default=False, help_text="Use SSL encryption for SMTP. Don't enable both TLS and SSL.")),
                ('timeout', models.IntegerField(default=30, help_text='Connection timeout in seconds')),
                ('debug_level', models.IntegerField(default=0, help_text='SMTP debug level (0-2). Higher values for more verbose logging.')),
            ],
            options={
                'verbose_name': 'Email Configuration',
                'verbose_name_plural': 'Email Configurations',
                'db_table': 'system_email_configuration',
                'permissions': [('add_email_configuration', 'Can add email configuration'), ('change_email_configuration', 'Can change email configuration'), ('delete_email_configuration', 'Can delete email configuration'), ('view_email_configuration', 'Can view email configuration')],
            },
            bases=('system_settings.systemsetting',),
        ),
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('systemsetting_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='system_settings.systemsetting')),
                ('html_template', models.TextField(default='<!DOCTYPE html>\n<html>\n<head>\n    <meta charset="utf-8">\n    <meta name="viewport" content="width=device-width, initial-scale=1.0">\n</head>\n<body>\n    <div>___header___</div>\n    <div>___main_content___</div>\n    <div>___footer___</div>\n</body>\n</html>', help_text='HTML template with placeholders like ___content1___, ___button1Title___')),
                ('content_placeholders', models.JSONField(default=list, help_text="List of content placeholders in the template (e.g., ['content1', 'button1Title'])")),
            ],
            options={
                'verbose_name': 'Email Template',
                'verbose_name_plural': 'Email Templates',
                'db_table': 'system_email_template',
                'permissions': [('add_email_template', 'Can add email template'), ('change_email_template', 'Can change email template'), ('delete_email_template', 'Can delete email template'), ('view_email_template', 'Can view email template')],
            },
            bases=('system_settings.systemsetting',),
        ),
        migrations.CreateModel(
            name='SystemEmail',
            fields=[
                ('systemsetting_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='system_settings.systemsetting')),
                ('available_variables', models.JSONField(default=list, help_text="Additional variables specific to this type of system email (e.g., ['order_number', 'amount']). Base variables are automatically included.")),
                ('cc', models.JSONField(default=list)),
                ('bcc', models.JSONField(default=list)),
                ('trigger_event', models.CharField(help_text='Event that triggers this email. Must be one of the predefined trigger events.', max_length=255)),
                ('template', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='system_settings.emailtemplate')),
            ],
            options={
                'verbose_name': 'System Email',
                'verbose_name_plural': 'System Emails',
                'db_table': 'system_email',
                'permissions': [('add_system_email', 'Can add system email'), ('change_system_email', 'Can change system email'), ('delete_system_email', 'Can delete system email'), ('view_system_email', 'Can view system email')],
            },
            bases=('system_settings.systemsetting',),
        ),
        migrations.CreateModel(
            name='EmailContent',
            fields=[
                ('systemsetting_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='system_settings.systemsetting')),
                ('language_code', models.CharField(max_length=10)),
                ('subject', models.CharField(max_length=255)),
                ('content_sections', models.JSONField(default=dict, help_text='Dict mapping template placeholders to content. Content can include any variables in {{ variable }} format.')),
                ('content_type', models.CharField(choices=[('text', 'text'), ('html', 'html')], default=shared_models.constants.ContentType['HTML'], max_length=20)),
                ('system_email', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='contents', to='system_settings.systememail')),
            ],
            options={
                'verbose_name': 'Email Content',
                'verbose_name_plural': 'Email Contents',
                'db_table': 'system_email_content',
                'permissions': [('add_email_content', 'Can add email content'), ('change_email_content', 'Can change email content'), ('delete_email_content', 'Can delete email content'), ('view_email_content', 'Can view email content')],
                'unique_together': {('system_email', 'language_code', 'content_type')},
            },
            bases=('system_settings.systemsetting',),
        ),
    ]



import logging
from django.conf import settings
from django.db import models

logger = logging.getLogger(__name__)

class SensitiveFieldsTrackingMixin:
    """
    Mixin to track changes in sensitive fields and log them appropriately.
    """
    sensitive_fields = []  # List of fields to track
    requires_approval_fields = []  # Fields that require approval before change

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._original_sensitive_values = self._get_sensitive_values()

    def _get_sensitive_values(self):
        """Get current values of sensitive fields"""
        return {
            field: getattr(self, field)
            for field in self.sensitive_fields
            if hasattr(self, field)
        }

    def _get_sensitive_field_changes(self):
        """Compare current values with original values"""
        current_values = self._get_sensitive_values()
        changes = {}
        
        for field, original_value in self._original_sensitive_values.items():
            current_value = current_values.get(field)
            if current_value != original_value:
                changes[field] = {
                    'from': original_value,
                    'to': current_value
                }
        return changes

    def _get_actor_info(self, request=None):
        """Get actor information from request"""
        if request and hasattr(request, 'user') and request.user and request.user.is_authenticated:
            # User is authenticated
            service_name = getattr(settings, 'SERVICE_NAME', '').lower()
            actor_details = {
                'email': getattr(request.user, 'email', None),
                'username': getattr(request.user, 'username', None),
                'first_name': getattr(request.user, 'first_name', None),
                'last_name': getattr(request.user, 'last_name', None)
            }
            logger.debug(f"Found authenticated user: {actor_details['email']}")
            
            if service_name == 'cp':
                return {
                    'type': 'CUSTOMER',
                    'id': request.user.id,
                    'details': actor_details
                }
            return {
                'type': 'USER',
                'id': request.user.id,
                'details': actor_details
            }
        
        logger.debug("No authenticated user found, using SYSTEM actor")
        return {
            'type': 'SYSTEM',
            'id': 0,
            'details': {'system': 'automated'}
        }

    def _get_affected_customer_id(self):
        """Get the affected customer ID"""
        # Import locally to avoid circular imports
        try:
            from shared_models.customers.models import Customer
        except ImportError:
            # Fallback if the import fails
            Customer = None
        
        # If this model is a Customer
        if Customer and isinstance(self, Customer):
            return self.id
            
        # If this model has a customer relation
        if hasattr(self, 'customer_id'):
            return self.customer_id
        if hasattr(self, 'customer'):
            return self.customer.id
            
        return None

    def save(self, *args, **kwargs):
        # Extract request from kwargs if present
        request = kwargs.pop('request', None)
        
        is_new = self._state.adding
        if not is_new:
            changes = self._get_sensitive_field_changes()
            if changes:
                self._handle_sensitive_field_changes(changes, request=request)
        
        result = super().save(*args, **kwargs)
        if not is_new:
            self._original_sensitive_values = self._get_sensitive_values()
        return result

    def _handle_sensitive_field_changes(self, changes, request=None):
        """Handle changes in sensitive fields"""
        # Get actor information using passed request if available
        actor_info = self._get_actor_info(request)
        affected_customer_id = self._get_affected_customer_id()
        
        logger.debug(f"Handling sensitive changes. Actor: {actor_info}, Customer: {affected_customer_id}")
        
        ip_address = None
        user_agent = None
        
        if request:
            # Get IP address
            x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
            if x_forwarded_for:
                ip_address = x_forwarded_for.split(',')[0]
            else:
                ip_address = request.META.get('REMOTE_ADDR')
                
            # Get user agent
            user_agent = request.META.get('HTTP_USER_AGENT')
            
            logger.debug(f"Request context found. IP: {ip_address}, UA: {user_agent}")
        else:
            logger.debug("No request context found")
        
        for field, change in changes.items():
            # Try to import AuditLogService, but make it optional
            try:
                from apps.common.modules.audit.services import AuditLogService
                # Create audit log entry with full context
                AuditLogService.log(
                    actor_type=actor_info['type'],
                    actor_id=actor_info['id'],
                    actor_details=actor_info['details'],
                    action='SENSITIVE_CHANGE',
                    description=f'Sensitive field {field} was changed',
                    customer_description=f'Your {field} information was updated',
                    target_object=self,
                    level='WARNING',
                    changes={
                        'field': field,
                        'changes': change,
                        'field_type': 'sensitive'
                    },
                    metadata={
                        'service': {
                            'name': getattr(settings, 'SERVICE_NAME', 'crm'),
                            'environment': getattr(settings, 'ENVIRONMENT', 'development')
                        },
                        'event': {
                            'dataset': 'sensitive_data',
                            'module': 'customer',
                            'field_name': field
                        },
                        'request': {
                            'ip_address': ip_address,
                            'user_agent': user_agent
                        }
                    },
                    affected_customer_id=affected_customer_id,
                    ip_address=ip_address,
                    user_agent=user_agent
                )
            except ImportError:
                # Log that audit service is not available
                logger.warning(f"AuditLogService not available. Sensitive field {field} was changed but not logged.")

            # If field requires approval, handle it
            if field in self.requires_approval_fields:
                self._handle_approval_required_change(field, change)

    def _handle_approval_required_change(self, field, change):
        """Handle changes that require approval"""
        try:
            from apps.common.modules.notifications.services import NotificationService
            
            # Create a notification for admins
            NotificationService.send_notification(
                type='SENSITIVE_DATA',
                subtype='CHANGE_APPROVAL_REQUIRED',
                title=f'Approval Required: {field} Change',
                message=f'A change to {field} requires approval',
                level='WARNING',
                data={
                    'field': field,
                    'changes': change,
                    'model': self._meta.model_name,
                    'object_id': self.pk
                }
            )
        except ImportError:
            # Log that notification service is not available
            logger.warning(f"NotificationService not available. Approval required for {field} change but notification not sent.") 


"""
IB Onboarding tracking models
"""
from django.db import models
from django.utils import timezone
from shared_models.common.base import BaseModel


class IBOnboardingStatus(BaseModel):
    """
    Track IB onboarding progress through the complete setup process.
    Ensures all required steps are completed before an IB can operate.
    """
    
    # Creation source choices
    CREATION_SOURCE_CHOICES = [
        ('CP', 'Client Portal - Client initiated'),
        ('CRM', 'CRM - Admin initiated'),
    ]
    
    customer = models.OneToOneField(
        'customers.Customer', 
        on_delete=models.CASCADE, 
        related_name='ib_onboarding_status',
        help_text="The IB customer being onboarded"
    )
    
    creation_source = models.CharField(
        max_length=10,
        choices=CREATION_SOURCE_CHOICES,
        null=True,
        blank=True,
        help_text="How the IB was created (CP client-initiated or CRM admin-initiated)"
    )
    
    # Step completion tracking
    ib_journey_form_submitted = models.BooleanField(
        default=False,
        help_text="IB has submitted the IB journey application form"
    )
    ib_account_created = models.BooleanField(
        default=False,
        help_text="IB account has been created in MT5"
    )
    hierarchy_created = models.BooleanField(
        default=False,
        help_text="IB has been positioned in hierarchy (standalone or under parent)"
    )
    agreement_assigned = models.BooleanField(
        default=False,
        help_text="IB has been assigned to at least one commission agreement"
    )
    referral_source_created = models.BooleanField(
        default=False,
        help_text="IB referral code has been created for client onboarding"
    )
    
    # Completion tracking
    completed_at = models.DateTimeField(
        null=True, 
        blank=True,
        help_text="Timestamp when all onboarding steps were completed"
    )
    completed_by = models.ForeignKey(
        'users.CRMUser',
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name='completed_ib_onboardings',
        help_text="CRM user who completed the onboarding"
    )
    
    # Store references to created entities
    ib_request_id = models.IntegerField(
        null=True, 
        blank=True,
        db_index=True,
        help_text="ID of the IB journey form submission (IBRequest from backend-py project)"
    )
    hierarchy = models.ForeignKey(
        'ib_commission.IBHierarchy', 
        null=True, 
        blank=True, 
        on_delete=models.SET_NULL,
        help_text="Reference to the created hierarchy entry"
    )
    agreement = models.ForeignKey(
        'ib_commission.IBAgreement', 
        null=True, 
        blank=True, 
        on_delete=models.SET_NULL,
        help_text="Primary agreement assigned to the IB"
    )
    referral_source = models.ForeignKey(
        'referrals.ReferralSource', 
        null=True, 
        blank=True, 
        on_delete=models.SET_NULL,
        help_text="Primary referral source created for the IB"
    )
    
    # Additional tracking
    notes = models.TextField(
        blank=True,
        help_text="Any notes or special instructions for this IB's onboarding"
    )
    
    class Meta:
        app_label = 'ib_commission'
        db_table = 'ib_onboarding_status'
        verbose_name = 'IB Onboarding Status'
        verbose_name_plural = 'IB Onboarding Statuses'
        indexes = [
            models.Index(fields=['customer']),
            models.Index(fields=['completed_at']),
            models.Index(fields=['ib_account_created', 'hierarchy_created', 
                               'agreement_assigned', 'referral_source_created']),
        ]
    
    def __str__(self):
        return f"IB Onboarding: {self.customer} - {'Completed' if self.is_completed else 'In Progress'}"
    
    @property
    def is_completed(self):
        """Check if all onboarding steps are completed"""
        if self.creation_source == 'CRM':
            # CRM-initiated IBs don't require the journey form
            return all([
                self.ib_account_created,
                self.hierarchy_created,
                self.agreement_assigned,
                self.referral_source_created
            ])
        else:
            # CP-initiated IBs require all steps including journey form
            return all([
                self.ib_journey_form_submitted,
                self.ib_account_created,
                self.hierarchy_created,
                self.agreement_assigned,
                self.referral_source_created
            ])
    
    @property
    def completion_percentage(self):
        """Calculate the percentage of completed steps"""
        if self.creation_source == 'CRM':
            # CRM path has 4 required steps
            steps = [
                self.ib_account_created,
                self.hierarchy_created,
                self.agreement_assigned,
                self.referral_source_created
            ]
        else:
            # CP path has 5 required steps
            steps = [
                self.ib_journey_form_submitted,
                self.ib_account_created,
                self.hierarchy_created,
                self.agreement_assigned,
                self.referral_source_created
            ]
        completed = sum(1 for step in steps if step)
        return (completed / len(steps)) * 100
    
    @property
    def next_step(self):
        """Determine the next required step in the onboarding process"""
        # For CRM-created IBs, skip the journey form requirement
        if self.creation_source != 'CRM' and not self.ib_journey_form_submitted:
            return 'ib_journey_form'
        elif not self.ib_account_created:
            return 'ib_account'
        elif not self.hierarchy_created:
            return 'hierarchy'
        elif not self.agreement_assigned:
            return 'agreement'
        elif not self.referral_source_created:
            return 'referral_source'
        else:
            return None
    
    @property
    def missing_steps(self):
        """Get a list of incomplete steps"""
        steps = []
        # For CRM-created IBs, don't include journey form as missing
        if self.creation_source != 'CRM' and not self.ib_journey_form_submitted:
            steps.append('ib_journey_form')
        if not self.ib_account_created:
            steps.append('ib_account')
        if not self.hierarchy_created:
            steps.append('hierarchy')
        if not self.agreement_assigned:
            steps.append('agreement')
        if not self.referral_source_created:
            steps.append('referral_source')
        return steps
    
    def mark_completed(self, user=None):
        """Mark the onboarding as completed if all steps are done"""
        if self.is_completed and not self.completed_at:
            self.completed_at = timezone.now()
            if user:
                self.completed_by = user
            self.save(update_fields=['completed_at', 'completed_by'])
            return True
        return False
    
    def update_step(self, step_name, completed=True, reference=None, user=None):
        """
        Update a specific onboarding step
        
        Args:
            step_name: One of 'ib_journey_form', 'ib_account', 'hierarchy', 'agreement', 'referral_source'
            completed: Whether the step is completed
            reference: The related object (ib_request, hierarchy, agreement, or referral_source)
            user: The user completing the step
        """
        update_fields = []
        
        if step_name == 'ib_journey_form':
            self.ib_journey_form_submitted = completed
            update_fields.append('ib_journey_form_submitted')
            if reference and hasattr(reference, 'id'):
                self.ib_request_id = reference.id
                update_fields.append('ib_request_id')
            
        elif step_name == 'ib_account':
            self.ib_account_created = completed
            update_fields.append('ib_account_created')
            
        elif step_name == 'hierarchy' and reference:
            self.hierarchy_created = completed
            self.hierarchy = reference
            update_fields.extend(['hierarchy_created', 'hierarchy'])
            
        elif step_name == 'agreement' and reference:
            self.agreement_assigned = completed
            self.agreement = reference
            update_fields.extend(['agreement_assigned', 'agreement'])
            
        elif step_name == 'referral_source' and reference:
            self.referral_source_created = completed
            self.referral_source = reference
            update_fields.extend(['referral_source_created', 'referral_source'])
        
        if update_fields:
            self.save(update_fields=update_fields)
            
            # Check if all steps are now completed
            if self.is_completed and not self.completed_at:
                self.mark_completed(user)
                
        return self.is_completed


# Generated by Django 4.2.20 on 2025-04-26 09:16

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CompanyBankAccount',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('created_by', models.CharField(blank=True, max_length=255, null=True)),
                ('updated_by', models.CharField(blank=True, max_length=255, null=True)),
                ('id', models.AutoField(editable=False, primary_key=True, serialize=False)),
                ('bank_name', models.CharField(max_length=255)),
                ('country', models.CharField(max_length=255)),
                ('currency', models.CharField(max_length=255)),
                ('account_number', models.CharField(max_length=255)),
                ('iban', models.CharField(max_length=255)),
                ('swift', models.CharField(max_length=255)),
                ('ifsc', models.CharField(max_length=255)),
                ('address', models.TextField(blank=True, default='')),
                ('account_holder_name', models.CharField(default='', max_length=255)),
            ],
            options={
                'verbose_name': 'Company Bank Account',
                'verbose_name_plural': 'Company Bank Accounts',
                'db_table': 'company_bank_accounts',
                'ordering': ['-created_at'],
                'permissions': [('view_company_bank_account', 'Can view company bank account'), ('delete_company_bank_account', 'Can delete company bank account'), ('change_company_bank_account', 'Can change company bank account'), ('add_company_bank_account', 'Can add company bank account')],
                'indexes': [models.Index(fields=['bank_name', 'country'], name='company_ban_bank_na_b2e9d5_idx'), models.Index(fields=['account_number'], name='company_ban_account_13481d_idx')],
            },
        ),
    ]



# Generated by Django 4.2.19 on 2025-04-18 16:40

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('transactions', '0006_transaction_approved_at'),
    ]

    operations = [
        migrations.AddField(
            model_name='transaction',
            name='assigned_user',
            field=models.ForeignKey(blank=True, help_text="The CRM user assigned to this transaction's customer at the time of creation.", null=True, on_delete=django.db.models.deletion.PROTECT, related_name='assigned_transactions', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddIndex(
            model_name='transaction',
            index=models.Index(fields=['assigned_user'], name='transaction_assigne_935da3_idx'),
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-15 16:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('customers', '0011_customerauth_reset_token_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='email',
            field=models.EmailField(max_length=254, unique=True, verbose_name='email address'),
        ),
        migrations.AlterField(
            model_name='customer',
            name='first_name',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AlterField(
            model_name='customer',
            name='last_name',
            field=models.CharField(blank=True, max_length=150),
        ),
        migrations.AlterField(
            model_name='customer',
            name='mobile',
            field=models.CharField(blank=True, max_length=20),
        ),
        migrations.AlterField(
            model_name='customer',
            name='phone',
            field=models.CharField(blank=True, max_length=20),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='address',
            field=models.CharField(blank=True, max_length=500),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='address2',
            field=models.CharField(blank=True, max_length=500),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='city',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='title',
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AlterField(
            model_name='customerdetail',
            name='zip_code',
            field=models.CharField(blank=True, max_length=20),
        ),
    ]



from django.db import models
from django.contrib.postgres.fields import ArrayField
from dataclasses import dataclass
from typing import List

@dataclass
class MT5ServerConfig:
    """Configuration for an MT5 server"""
    id: int
    name: str
    type: str  # 'demo' or 'live'
    ips: List[str]  # List of IP addresses for failover
    username: int
    manager_password: str
    api_password: str

    def __post_init__(self):
        """Validate configuration after initialization"""
        if self.type not in ['demo', 'live']:
            raise ValueError(f"Invalid server type: {self.type}. Must be 'demo' or 'live'")
        
        if not self.username:
            raise ValueError("Username must be provided")

        if not isinstance(self.username, int):
            raise ValueError("Username must be an integer")

        if not self.manager_password:
            raise ValueError("Manager password must be provided")

        if not self.ips:
            raise ValueError("At least one IP address must be provided")
        
        if not isinstance(self.id, int):
            raise ValueError("Server ID must be an integer")

class TradingPlatformServer(models.Model):
    PLATFORM_TYPES = (
        ('mt5', 'MetaTrader 5'),
        ('mt4', 'MetaTrader 4'),
        ('vertex', 'Vertex'),
    )

    SERVER_TYPES = (
        ('demo', 'Demo'),
        ('live', 'Live'),
    )

    name = models.CharField(max_length=255, unique=True)
    platform = models.CharField(max_length=10, choices=PLATFORM_TYPES)
    ips = ArrayField(
        models.CharField(max_length=15),
        help_text="List of IP addresses for the server"
    )
    type = models.CharField(max_length=4, choices=SERVER_TYPES)
    username = models.IntegerField(help_text="Platform Manager account login number")
    is_active = models.BooleanField(default=True)
    is_connected = models.BooleanField(default=False)
    connection_timeout = models.IntegerField(default=30)
    max_retries = models.IntegerField(default=3)
    retry_min_wait = models.IntegerField(default=1)
    retry_max_wait = models.IntegerField(default=30)
    health_check_interval = models.IntegerField(default=30)
    error_log = models.TextField(blank=True, null=True)
    description = models.TextField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = 'system_trading_platform_server'
        app_label = 'system_settings'
        verbose_name = "Trading Platform Server"
        verbose_name_plural = "Trading Platform Servers"
        permissions = [
            ('view_trading_platform_server', 'Can view trading platform server'),
            ('add_trading_platform_server', 'Can add trading platform server'),
            ('change_trading_platform_server', 'Can change trading platform server'),
            ('delete_trading_platform_server', 'Can delete trading platform server'),
            ('enable_trading_platform_server', 'Can enable trading platform server'),
            ('disable_trading_platform_server', 'Can disable trading platform server'),
            ('view_trading_platform_config', 'Can view trading platform config'),
        ]

    def __str__(self):
        return f"{self.name} ({self.platform} - {self.type})"

    def save(self, *args, **kwargs):
        # Ensure type is lowercase before saving
        if self.type:
            self.type = self.type.lower()
        super().save(*args, **kwargs)
    
    def to_server_config(self):
      """Convert to MT5ServerConfig object"""
      # Create and return MT5ServerConfig object directly
      return MT5ServerConfig(
          id=int(self.id),
          name=str(self.name),
          ips=[str(ip) for ip in self.ips],
          type=str(self.type.lower()),
          username=int(self.username),
          manager_password=str(self.manager_password),
          api_password=str(self.api_password)
      )

class TradingPlatformServerLog(models.Model):
    """Stores historical logs for trading platform server events."""

    EVENT_TYPES = (
        ('CONNECT_SUCCESS', 'Connection Successful'),
        ('CONNECT_FAIL', 'Connection Failed'),
        ('DISCONNECT', 'Disconnected'),
        ('HEALTH_CHECK_FAIL', 'Health Check Failed'),
        ('RECONNECT_ATTEMPT', 'Reconnection Attempt'),
        ('RECONNECT_SUCCESS', 'Reconnection Successful'),
        ('RECONNECT_FAIL', 'Reconnection Failed'),
    )

    server = models.ForeignKey(
        TradingPlatformServer,
        on_delete=models.CASCADE,
        related_name='logs'
    )
    timestamp = models.DateTimeField(auto_now_add=True, db_index=True)
    event_type = models.CharField(max_length=20, choices=EVENT_TYPES, db_index=True)
    details = models.TextField(blank=True, null=True, help_text="Details about the event, e.g., error message.")

    class Meta:
        db_table = 'system_trading_platform_server_log'
        app_label = 'system_settings' # Ensure this matches the app label of TradingPlatformServer
        verbose_name = "Trading Platform Server Log"
        verbose_name_plural = "Trading Platform Server Logs"
        ordering = ['-timestamp']

    def __str__(self):
        return f"{self.timestamp} - {self.server.name} - {self.event_type}"



# Generated by Django 4.2.19 on 2025-03-18 10:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0012_alter_crmuser_managers_usersettings_is_2fa_enabled_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='usersettings',
            name='is_2fa_enabled',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='usersettings',
            name='two_factor_secret',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



from django.apps import AppConfig

class UsersConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'shared_models.users'
    verbose_name = 'Shared Users'
    label = 'users'
    
    def ready(self):
        print("DEBUG: Shared Users app is being loaded!")


# Generated by Django 4.2.20 on 2025-04-10 10:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('transactions', '0004_alter_transaction_created_by'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='status',
            field=models.CharField(choices=[('PENDING', 'Pending'), ('APPROVED', 'Approved'), ('REJECTED', 'Rejected'), ('CANCELLED', 'Cancelled'), ('FAILED', 'Failed'), ('PROCESSED', 'Processed')], default='PENDING', max_length=10),
        ),
    ]



from django.apps import AppConfig


class UIElementsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    # Define the app label explicitly, matching the models
    name = 'shared_models.ui_elements' 
    label = 'ui_elements' # This label must match the app_label in models.py
    verbose_name = "UI Elements (Shared)" # Optional: human-readable name 


# Generated by Django 4.2.19 on 2025-02-25 19:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('customers', '0005_customerauth_temp_webauthn_challenge'),
    ]

    operations = [
        migrations.AddField(
            model_name='customerauth',
            name='webauthn_challenge',
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
    ]



# Generated by Django 5.1.4 on 2025-01-23 11:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0009_alter_userprofile_profile_picture'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='crmuser',
            options={'permissions': [('change_access_crmuser', 'Can block/unblock user'), ('reset_password_crmuser', 'Can reset user passwords'), ('can_assign_users', 'Can assign users to customers')], 'verbose_name': 'user', 'verbose_name_plural': 'users'},
        ),
    ]



# Generated by Django 5.1.4 on 2025-01-09 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0008_usersettings_temp_webauthn_challenge'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='profile_picture',
            field=models.CharField(blank=True, max_length=500, null=True),
        ),
    ]
